/*
 * Written in 2021 by Magnus Bergman <magnus@snisurset.net>
 *
 * To the extent possible under law, the author(s) have dedicated all
 * copyright and related and neighboring rights to this software to the
 * public domain worldwide.
 *
 * This software is distributed without any warranty.
 *
 * See: http://creativecommons.org/publicdomain/zero/1.0/
 */
#include <cairo-svg.h>
#include <fcntl.h>                      /* open() */
#include <stddef.h>                     /* NULL */
#include <stdio.h>
#include <sys/mman.h>                   /* mmap() */
#include <sys/stat.h>
#include <unistd.h>                     /* close() */

#include "cgmdraw-cairo.h"


static int
_mmap_open(const char *filename, int *r_fd,const void **r_data, size_t *r_len)
{
    struct stat statbuf;

    if(stat(filename,&statbuf))
        return -1;

    *r_fd = open(filename,O_RDONLY);
    if(*r_fd<0)
        return -1;
    *r_data = mmap(NULL,statbuf.st_size,PROT_READ,MAP_SHARED|MAP_POPULATE,*r_fd,0);
    if(MAP_FAILED==*r_data){
        close(*r_fd);
        return -1;
    }
    *r_len = statbuf.st_size;
    return 0;
}


static void
_mmap_close(int fd,const void *data,size_t len)
{
    munmap((void*)data,len);
    close(fd);
}


typedef struct{
    cairo_surface_t *surface;
    const char *filename;
}context_t;


static cairo_t *
on_page(void *userdata, int width, int height){
    context_t *c = userdata;
    c->surface = cairo_svg_surface_create(c->filename, width, height);
    return cairo_create(c->surface);
}


int main(int argc, char **argv){
    int fd;
    const void *data;
    size_t len;
    context_t c;
    if(argc < 3){
        printf("Usage: %s <filename.cgm> <filename.svg>", argv[0]);
        return 1;
    }
    if(_mmap_open(argv[1], &fd, &data, &len)){
        printf("Failed to read %s", argv[1]);
        return 1;
    }

    c.surface = NULL;
    c.filename = argv[2];
    cgm_draw_cairo(data, len, on_page, &c);
    cairo_surface_mark_dirty(c.surface);
    cairo_surface_destroy(c.surface);

    _mmap_close(fd, data, len);
    return 0;
}
