/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#include "abydos-version.h"

/**
 * SECTION: abydos-version
 * @short_description: Compile-time and run-time version checks.
 * @title: Version Information
 * @include: abydos.h
 **/

/**
 * abydos_version
 *
 * The run-time version of abydos, encoded using ABYDOS_ENCODE_VERSION().
 */
int
abydos_version()
{
    return ABYDOS_VERSION;
}


/**
 * abydos_version_major
 *
 * The run-time major version of abydos.
 */
int
abydos_major_version()
{
    return ABYDOS_MAJOR_VERSION;
}


/**
 * abydos_version_minor
 *
 * The run-time minor version of abydos.
 */
int
abydos_minor_version()
{
    return ABYDOS_MINOR_VERSION;
}


/**
 * abydos_micro_version
 *
 * The run-time micro version of abydos.
 */
int
abydos_micro_version()
{
    return ABYDOS_MICRO_VERSION;
}


/**
 * abydos_version_string
 *
 * The run-time version of abydos as a string.
 */
const char *
abydos_version_string()
{
    return ABYDOS_VERSION_STRING;
}


/**
 * abydos_check_version
 * @major: the major version to check for
 * @minor: the minor version to check for
 * @micro: the micro version to check for
 *
 * Checks the run-time version of abydos.
 * Use ABYDOS_CHECK_VERSION() for a compile-time version check.
 *
 * returns: non zero if the run-time version of abydos is the same as
 *          or newer than the passed-in version.
 **/
int
abydos_check_version(int major,int minor,int micro)
{
    return ABYDOS_ENCODE_VERSION(major,minor,micro) >= ABYDOS_VERSION;
}
