/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef memutil_h
#define memutil_h

#ifdef FRIENDUP
# include <core/types.h>
# define u_new(a,b) (a*)FMalloc(sizeof(a)*(b))
# define u_new0(a,b) (a*)FCalloc(b,sizeof(a))
# define u_free FFree
# define u_strdup(a) StringDuplicate(a)
# define u_strndup(a,b) StringDuplicateN(a,b)
#elif defined(USE_JEMALLOC)
# include <jemalloc/jemalloc.h>
# include <string.h>                    /* memcpy(), memset() */
# define u_new(a,b) (a*)malloc(sizeof(a)*(b))
# define u_new0(a,b) (a*)mallocx(sizeof(a)*(b),MALLOCX_ZERO)
# define u_malloc(a) mallocx(a,0)
# define u_renew(a,b,c) ((b)?(a*)rallocx(b,sizeof(a)*(c),0):(a*)mallocx(sizeof(a)*(c),0))
/* # define u_free(a) dallocx(a,0) */
# define u_free free
# define u_strdup(a) ((a)?strdup(a):NULL)
# define u_strndup(a,b) ((a)?strndup(a,b):NULL)
#elif defined(USE_GLIB)
# include <glib.h>
# define u_new g_new
# define u_new0 g_new0
# define u_malloc g_malloc
# define u_realloc g_realloc
# define u_renew(a,b,c) ((b)?g_renew(a,b,c):(a*)g_malloc(sizeof(a)*(c)))
# define u_free g_free
# define u_strdup g_strdup
# define u_strndup g_strndup
#else
# include <stdlib.h>                    /* malloc() */
# ifdef HELENOS
#  include <mem.h>                      /* memset() */
# else
#  include <string.h>                   /* memset() */
# endif
# define u_new(a,b) (a*)malloc(sizeof(a)*(b))
# define u_new0(a,b) (a*)memset(malloc(sizeof(a)*(b)),0,sizeof(a)*(b))
# define u_malloc malloc
# define u_realloc realloc
# define u_renew(a,b,c) ((b)?(a*)realloc(b,sizeof(a)*(c)):(a*)malloc(sizeof(a)*(c)))
# define u_free free
# define u_strdup(a) strdup(a)
# define u_strndup(a,b) strndup(a,b)
#endif

#define u_free_if_set(a) if(a) u_free(a)
#define u_strdup_safe(a) ((a)?u_strdup(a):NULL)
#define u_strndup_safe(a,b) ((a)?u_strndup(a,b):NULL)

#ifndef NULL
# define NULL ((void*)0)
#endif

#endif /* memutil_h */
